package kraft.gateway;

import java.io.*;
import java.net.*;


public class GatewayMap
{
private static final int default_size=10;		//default table size
private static final String default_protocol="socket";	//default protocol
public int table_size;

GatewayEntry[] table;


//
// constructor w/ a default table size
//
public GatewayMap()
	{
	this(default_size);
	}

//
// constructor w/ a customised table size
//
public GatewayMap(int size)
	{
	table=new GatewayEntry[size];		//create an array of default size
	for (int i=0;i<size;i++)
		table[i]=null;			//init all entries
	this.table_size=size;
	}


//
// add a gateway entry
// given the site, protocol, host & port
//
public void add_gateway(String site,String host,int port)
	throws Exception, MalformedURLException
	{
	GatewayEntry entry;

	entry=new GatewayEntry(site,host,port);			//create an entry
	add_gateway(site,entry);
	}

public void add_gateway(String site,String protocol,String host,int port)
	throws Exception, MalformedURLException
	{
	GatewayEntry entry;

	entry=new GatewayEntry(site,protocol,host,port);	//create an entry
	add_gateway(site,entry);
	}


//
// add a gateway entry
// given the site & entry object
//
public  void add_gateway(String site,GatewayEntry entry)
	throws Exception
	{
	try	{
		int i=0;

		delete_gateway(entry.get_site());		//remove any old entry
		while (table[i]!=null)				//scan for an empty slot
			i++;
		table[i]=entry;					//put entry there
		} catch (IndexOutOfBoundsException e)		//catch out of bound
			{
			throw new Exception("table overflow");
			}
	}





//
// find the gateway entry given the site name
//
public GatewayEntry find_gateway(String s)
	{
	int i=0;
	try	{
		while (true)
			{
			if (table[i]==null)
				i++;
			else if (!table[i].get_site().equals(s))	//loop until the site is found
					i++;
				else break;
			}
		} catch (IndexOutOfBoundsException e)
			{
			return(null);			//cannot find it
			}
	return(table[i]);
	}

//
// remove a gateway entry base on the site
//
public boolean delete_gateway(String s)
	{
	int i=0;
	try	{
		while (true)
			{
			if (table[i]==null)
				i++;
			else if (!table[i].get_site().equals(s))
					i++;
				else break;
			}
		} catch (IndexOutOfBoundsException e)
			{
			return(false);			//cannot find it
			}
	table[i]=null;					//remove entry
	return(true);
	}


//
// get all site names
//
public String[] get_all_sites()
	{
	String[] temp;
	int i,j;
	int count=0;

	//
	// count no. of non-empty entries
	//
	for (i=0;i<table.length;i++)
		{
		if (table[i]!=null)
			{
			count++;
			}
		}

	//
	// create array and copy site names over
	//
	temp=new String[count];
	j=0;
	for (i=0;i<table.length;i++)
		{
		if (table[i]!=null)
			{
			temp[j]=table[i].get_site();
			j++;
			}
		}
	return(temp);
	}
}


