package kraft.monitor.display;

import java.util.Date;


/** Used to allow the MonitorGUI to 'know' when a particular event has been
  * displayed by the MonitorCanvas. 
  *<br><br>
  *
  * Makes sure that the MonitorGUI knows which step this event occurs at,
  * and when it has been displayed, to ensure that the event currently shown 
  * in the visualisation is the most recent event in the agent system being
  * visualised. <br><br>
  *
  * The MonitorCanvas will set the index position of this event in its 
  * sequence of events, and will assign it to a particular DisplayEvent object,
  * the event object will then inform this DisplayMonitor when the event has 
  * been displayed graphically.
  *
  *@author Ted Francis
  *@see kraft.monitor.display.MonitorCanvas#displayMessage
  *@see kraft.monitor.display.DisplayEvent
  *@see kraft.monitor.display.MonitorCanvas
  *@see kraft.monitor.display.MonitorGUI
  */

//===========================
public class DisplayMonitor {
//===========================


  public DisplayMonitor(Date t) {
    time      = t;
    index     = -1;
    displayed = false;
  }

  private boolean  displayed;
  private Date     time;
  private int      index;
  
  /** Called by the MonitorCanvas */
  public void setIndex(int s) {
    index = s;
  }

  /** Called by the MonitorGUI */
  public int getIndex() {
    return index;
  }

  /** Set to true by the DisplayEvent when the event is displayed*/
  public void setDisplayed(boolean val) {
    displayed = val;
  }

  /** Called by the MonitorGUI to see whether the event is displayed yet */
  public boolean isDisplayed() {
    return displayed;
  }

  public Date getTime(){
    return time;
  }
    
  public String toString() {
    return "DisplayMonitor: [index = "+index+", displayed = "+ displayed+", time = " + time + "]";
  }
  
}//end class DisplayMonitor














