package kraft.monitor.display;

import java.rmi.*;


/** Defines the methods that the Monitor can provide to 
  * agents wishing to be monitored.
  * These methods are remotely accessible, and recognise
  * any agent and any message object. (Via the use of 
  * the Monitorable and Message interfaces)
  *
  *@author Ted Francis
  *@see Monitorable
  *@see Message
  */

public interface MonitorServer extends Remote {

  
  /** Inform the Monitor that the Monitorable object with name 'n' 
    * does not wish to be monitored further
    */
  public void deRegister(String n) 
    throws RemoteException;


  /** Inform the Monitor that the given Monitorable wishes to have its
    * activity monitored.
    */
  public void register(Monitorable a) 
    throws RemoteException;


  /** Inform the monitor of a message being passed between two agents
    * with the names given.
    */
  public void informOfMessage(String from, String to, Message m) 
    throws RemoteException;
  

  /** Inform the monitor of a message that has been sent and has not
    * been delivered successfully.
    * The Monitor should already have been informed of this Message 
    * by a call to informOfMessage.
    */
  public void informOfFailure(String from, String to, Message m)
    throws RemoteException;


  /** The agent called 'name' wishes to register its current status
    * with the 'comment' string.
    */
  public void informOfStatus(String name, String comment)
    throws RemoteException;

}//end interface MonitorServer














